/**
   This is a simple LoRa device which exposes two properties:

   input property:    S (values 0 and 1) to control the attached LED
   output property:   B which is 0 when the attached button is released and 1 when it is pressed

*/
/**
   This code (The Software) is supplied together with "MQTT Manager, Lora and Lora 'Poor Man' Gateway" Instructable and it is supplied to you
   by the Author in consideration of your agreement to the following terms, and your use or installation of The Software constitutes acceptance of these terms.
   If you do not agree with these terms, please do not use or install The Software.
   The Author grants you a personal, non-exclusive license, under authors copyrights in this original software, to use The Software.
   Except as expressly stated in this notice, no other rights or licenses, express or implied, are granted by the Author, including but not limited to any
   patent rights that may be infringed by your derivative works or by other works in which The Software may be incorporated.
   The Software is provided by the Author on an AS IS basis.  THE AUTHOR MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT
   LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND OPERATION
   ALONE OR IN COMBINATION WITH YOUR PRODUCTS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
   REPRODUCTION AND MODIFICATION OF THE SOFTWARE HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
   STRICT LIABILITY OR OTHERWISE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**/

#include <SPI.h>
#include <LoRa.h>    // https://github.com/sandeepmistry/arduino-LoRa

#define DEBUG         //If you comment this line, the DPRINT & DPRINTLN lines are defined as blank.

#ifdef DEBUG
#define DBEGIN(x)      Serial.begin(x)
#define DPRINT(...)    Serial.print(__VA_ARGS__)
#define DPRINTLN(...)  Serial.println(__VA_ARGS__)
#else
#define DBEGIN(x)
#define DPRINT(...)
#define DPRINTLN(...)
#endif

#define LORA_CS     5
#define LORA_RESET  17
#define LORA_IRQ    15

#define PUSHBUTTON_PIN  26
#define LED_PIN         27

String outgoing;              // outgoing message
byte msgCount = 0;            // count of outgoing messages
byte localAddress = 0xAB;     // address of this device
byte destination  = 0xCC;     // gateway address

void setup() {
  DBEGIN(115200);

  DPRINTLN("LoRa Device");

  LoRa.setPins(LORA_CS, LORA_RESET, LORA_IRQ);

  // initialize radio at 868 MHz
  if (!LoRa.begin(868E6)) {
    DPRINTLN("LoRa init failed. Check your connections.");
    while (true)
      ;
  }

  LoRa.onReceive(onReceive);
  LoRa.receive();
  DPRINTLN("LoRa init succeeded.");

  pinMode(PUSHBUTTON_PIN, INPUT);
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LOW);

  // Send initial state
  sendLoRaMessage("S", "0");
  sendLoRaMessage("B", "0");

  LoRa.receive();

  DPRINTLN("Device Ready");
}

void loop() {

  if (digitalRead(PUSHBUTTON_PIN)) {
    DPRINTLN("Pushed !");
    sendLoRaMessage("B", "1");
    LoRa.receive();
    while (digitalRead(PUSHBUTTON_PIN))
      ;
    DPRINTLN("Released !");
    sendLoRaMessage("B", "0");
    LoRa.receive();
  }
}

void sendLoRaMessage(String property, String value) {
  String outgoing = property + "=" + value;
  LoRa.beginPacket();                   // start packet
  LoRa.write(destination);              // add destination address
  LoRa.write(localAddress);             // add sender address
  LoRa.write(msgCount);                 // add message ID
  LoRa.write(outgoing.length());        // add payload length
  LoRa.print(outgoing);                 // add payload
  LoRa.endPacket();                     // finish packet and send it
  msgCount++;                           // increment message ID
}

void onReceive(int packetSize) {
  if (packetSize == 0) // if there's no packet, return
    return;

  // read packet header bytes:
  int recipient = LoRa.read();          // recipient address
  byte sender = LoRa.read();            // sender address
  byte incomingMsgId = LoRa.read();     // incoming msg ID
  byte incomingLength = LoRa.read();    // incoming msg length

  // if the recipient isn't this device or broadcast,
  if (recipient != localAddress && recipient != 0xFF) {
    DPRINTLN("This message is not for me.");
    return;                             // skip rest of function
  }

  String incoming = "";                 // payload of packet

  while (LoRa.available()) {            
    incoming += (char)LoRa.read();   
  }

  if (incomingLength != incoming.length()) {   // check length for error
    DPRINTLN("error: message length does not match length");
    return;                             
  }

  // if message is for this device, or broadcast, print details:
  DPRINTLN("Received from: 0x" + String(sender, HEX));
  DPRINTLN("Sent to: 0x" + String(recipient, HEX));
  DPRINTLN("Message ID: " + String(incomingMsgId));
  DPRINTLN("Message length: " + String(incomingLength));
  DPRINTLN("Message: " + incoming);
  DPRINTLN("RSSI: " + String(LoRa.packetRssi()));
  DPRINTLN();

  char buf[sizeof(incoming)];
  incoming.toCharArray(buf, sizeof(buf));
  char *property = strtok(buf, "=");
  char *val = strtok(NULL, "=");

  if (strcmp(property, "S") == 0) {
    if (strcmp(val, "1") == 0)
      digitalWrite(LED_PIN, HIGH);
    else
      digitalWrite(LED_PIN, LOW);
  }
}
